#include <windows.h>

#include <fstream>
#include <iostream>
#include <string>

using namespace std;

void replace_all ( std::string & str, std::string const & pattern, std::string const & replacement ) {

	std::string::size_type start = str.find( pattern, 0 );

	while ( start != str.npos ) {
		str.replace( start, pattern.size(), replacement );
		start = str.find( pattern, start+replacement.size() );
	}

}

void show_help() {

	cout << "Wirk2 - Windows Release Kit for Greenstone2" << endl;
	cout << "Helps you to create releases from the Repository" << endl << endl;

	cout << "usage: wirk2 [args]" << endl;
	cout << "	-sim" << endl;
	cout << "		simulation only, don't actually do anything" << endl << endl;

	cout << "	-help" << endl;
	cout << "		show this help screen" << endl << endl;

	cout << "	-from <target>" << endl;
	cout << "		start execution from the target with the given target address" << endl << endl;

	cout << "	-to <target>" << endl;
	cout << "		stop execution just before the target with the given target address" << endl << endl;

	cout << "	-descend <target>" << endl;
	cout << "		execute only the target with the given address, including subtargets" << endl << endl;

	cout << "	-cp" << endl;
	cout << "		show the classpath being used by wirk2" << endl << endl;

}


int main(int argc, char** argv ) {

	string classpath, command;
	
	//some checks
	bool ok = true;
	if ( getenv( "JAVA_HOME" ) == NULL ) {
		cerr << "Please set JAVA_HOME before running wirk2" << endl;
		ok = false;
	}
	if ( getenv( "WIRK2_HOME" ) == NULL ) {
		cerr << "Please set WIRK2_HOME before running wirk2" << endl;
		ok = false;
	}
	if ( !ok ) {
		return -1;
	}
	//string PWD = getenv( "CD" );

	//load environment variables
	string JAVA_HOME = getenv( "JAVA_HOME" );
	string WIRK2_HOME = getenv( "WIRK2_HOME" );

	//get the pwd
	string pwd;
	system( "CD > cd.dat" );
	ifstream file( "cd.dat" ) ;
	getline(file, pwd);
	file.close();
	system( "del cd.dat" );
		
	//set the classpath
	//classpath = pwd + "\\installer\\classes;" + JAVA_HOME + "\\core\\lib\\tools.jar;" + WIRK2_HOME + "\\core\\lib\\serializer.jar;" + WIRK2_HOME + "\\core\\lib\\xalan.jar;" + WIRK2_HOME + "\\core\\lib\\xercesImpl.jar;" + WIRK2_HOME + "\\core\\lib\\xml-apis.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-antlr.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-apache-bcel.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-apache-bsf.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-apache-log4j.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-apache-oro.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-apache-regexp.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-apache-resolver.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-commons-logging.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-commons-net.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-icontract.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-jai.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-javamail.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-jdepend.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-jmf.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-jsch.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-junit.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-launcher.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-netrexx.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-nodeps.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-starteam.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-stylebook.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-swing.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-trax.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-vaj.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-weblogic.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-xalan1.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\ant-xslp.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\xercesImpl.jar;" + WIRK2_HOME + "\\core\\ant\\lib\\xml-apis.jar;" + WIRK2_HOME + "\\core\\ant-installer\\lib\\ai-icons-amaranth.jar;" + WIRK2_HOME + "\\core\\ant-installer\\lib\\ai-icons-bluecurve.jar;" + WIRK2_HOME + "\\core\\ant-installer\\lib\\ai-icons-crystalsvg.jar;" + WIRK2_HOME + "\\core\\ant-installer\\lib\\ai-icons-eclipse.jar;" + WIRK2_HOME + "\\core\\ant-installer\\lib\\ai-icons-krystaline.jar;" + WIRK2_HOME + "\\core\\ant-installer\\lib\\ant-installer-ext.jar;" + WIRK2_HOME + "\\core\\ant-installer\\lib\\ant-installer.jar;" + WIRK2_HOME + "\\core\\ant-installer\\lib\\jgoodies-edited-1_2_2.jar;" + WIRK2_HOME + "\\core\\ant-installer\\lib\\xercesImpl.jar;" + WIRK2_HOME + "\\core\\ant-installer\\lib\\xml-apis.jar;" + WIRK2_HOME + "\\core\\ant-installer\\classes";
	//putenv( ("CLASSPATH=" + classpath).c_str() );

	//check that it worked
	//string changed_classpath = getenv("CLASSPATH");
	//cout << "classpath: " << changed_classpath << endl;
	
	//create the command
	command = WIRK2_HOME + "\\core\\ant\\bin\\ant.bat -f " + WIRK2_HOME + "\\ant-scripts\\build.xml -addressing -Dwirk2.home=" + WIRK2_HOME;
	//pass on the arguments
	string a;
	bool simMode = false;
	for ( int i=0; i < argc; i++ ) {
		a = argv[i];
			
		if ( a.compare("-help") == 0 || a.compare("--help") == 0 ) {
			show_help();
			return 0;
		} else if ( a.compare("-cp") == 0 || a.compare("--cp") == 0 ) {
			cout << classpath;
			return 0;
		} else if ( a.compare("-sim") == 0) {
			command = command + " " + a;
			simMode = true;
		} else {
			command = command + " " + a;
		}

	}

	//use forward slashes for everything else
	//replace_all( pwd, "\\", "/" );
	
	//set the basedir in the command
	command = command + " -Dbasedir=";
	command = command + pwd;


	
	//command = command + ".";

	cout
		<< "O-----------------------------------------O" << endl
		<< "|                                         |" << endl
		<< "|   WiRK2                                 |" << endl
		<< "|   Windows Release Kit for Greenstone2   |" << endl
		<< "|                                         |" << endl
		<< "O-----------------------------------------O" << endl
		;
	
	cout << "pwd: " << pwd << endl;
	cout << "command: " << command << endl;

	system( command.c_str() );
	
	return 0;

}
